package org.implab.gradle.containers;

import java.util.Optional;

import org.gradle.api.Project;
import org.gradle.api.file.DirectoryProperty;
import org.gradle.api.file.ProjectLayout;
import org.gradle.api.file.RegularFileProperty;
import org.gradle.api.model.ObjectFactory;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;

public class ContainerExtension {

    private final Property<String> cliCmd;

    private final Property<String> imageAuthority;

    private final Property<String> imageGroup;

    private final Property<String> imageShortName;

    private final Property<String> imageTag;

    private final Property<ImageName> imageName;

    private final DirectoryProperty contextDir;

    private final RegularFileProperty imageIdFile;

    public ContainerExtension(ObjectFactory factory, ProjectLayout layout, Project project) {
        contextDir = factory.directoryProperty();
        contextDir.convention(layout.getBuildDirectory().dir("context"));

        imageIdFile = factory.fileProperty();
        imageIdFile.convention(layout.getBuildDirectory().file("imageId"));

        cliCmd = factory.property(String.class);
        cliCmd.set("docker");

        imageAuthority = factory.property(String.class);
        imageGroup = factory.property(String.class);
        imageShortName = factory.property(String.class);

        imageShortName.convention(project.getName());

        imageTag = factory.property(String.class);
        imageTag.set(project
                .provider(() -> Optional.ofNullable(project.getVersion()).map(v -> v.toString()).orElse("latest")));

        Provider<String> imageRepository = imageGroup.map(g -> g + "/" + imageShortName.get()).orElse(imageShortName);

        imageName = factory.property(ImageName.class);
        imageName.convention(project.provider(
                () -> new ImageName().authority(imageAuthority.get()).name(imageRepository.get()).tag(imageTag.get())));
    }

    public Property<String> getCliCmd() {
        return cliCmd;
    }

    public void setCliCmd(String cliCmd) {
        this.cliCmd.set(cliCmd);
    }

    public DirectoryProperty getContextDirectory() {
        return contextDir;
    }

    public RegularFileProperty getImageIdFile() {
        return imageIdFile;
    }

    public Property<String> getImageAuthority() {
        return imageAuthority;
    }

    public Property<String> getImageGroup() {
        return imageGroup;
    }

    public Property<ImageName> getImageName() {
        return imageName;
    }

    public Property<String> getImageShortName() {
        return imageShortName;
    }

    public ImageName createImageName() {
        return new ImageName();
    }

}